#include"ModelSimulation.h"

RKModelSimulation::RKModelSimulation(Model *m,double step_size)
{
	model=m;
	t=0.0;
	Tm=step_size;
}

RKModelSimulation::~RKModelSimulation(void)
{
	delete model;
}

void RKModelSimulation::Step(void)
{
	Vector st=model->State();
	Vector k1=Tm*model->Evaluate(st,t);
	Vector k2=Tm*model->Evaluate(st+k1/2.0,t+Tm/2);
	Vector k3=Tm*model->Evaluate(st+k2/2.0,t+Tm/2);
	Vector k4=Tm*model->Evaluate(st+k3,t+Tm);
   	st=st+(k1+2.0*k2+2.0*k3+k4)/6.0;
   	model->State(st);
   	t+=Tm;
}

const Model* RKModelSimulation::ModelP(void) const
{
	return model;
}

double RKModelSimulation::Time(void) const
{
		return t;
}

double RKModelSimulation::SamplePeriod(void) const
{
		return Tm;
}

BaseModel::BaseModel(unsigned order)
    : state(order)
{
}

Vector BaseModel::State(void) const
{
	return state;
}

void BaseModel::State(const Vector &st)
{
	state=st;
}

double& BaseModel::State(unsigned i)
{
	return state[i];
}

double BaseModel::State(unsigned i) const
{
	return state[i];
}
