#ifndef MODELSIMULATION_H
#define MODELSIMULATION_H

#include<valarray>

typedef std::valarray<double> Vector;

class Model;

class ModelSimulation
{
  public:
  	virtual ~ModelSimulation(void) {};
  	virtual void Step(void)=0;
  	virtual const Model* ModelP(void) const=0;
  	virtual double Time(void) const=0;
  	virtual double SamplePeriod(void) const=0;
};

class RKModelSimulation : public ModelSimulation
{
	Model *model;
	double t;
	double Tm;
  public:
  	RKModelSimulation(Model *m,double step_size);
  	virtual ~RKModelSimulation(void);
  	virtual void Step(void);
  	virtual const Model* ModelP(void) const;
  	virtual double Time(void) const;
  	virtual double SamplePeriod(void) const;
};

class Model
{
  public:
  	virtual ~Model(void) {}
  	virtual Vector Evaluate(const Vector &state,double t) const=0;
  	virtual Vector State(void) const=0;
  	virtual double& State(unsigned i)=0;
  	virtual double State(unsigned i) const=0;
  	virtual void State(const Vector &state)=0;
};

class BaseModel : public Model
{
    Vector state;
  public:
    explicit BaseModel(unsigned order);
  	virtual Vector Evaluate(const Vector &state,double t) const=0;
  	virtual Vector State(void) const;
  	virtual double& State(unsigned i);
  	virtual double State(unsigned i) const;
  	virtual void State(const Vector &state);
};

#endif
